#!/bin/sh
#
# CSS Launcher for KEK
#
# Supported OS:
#  Linux
#  Mac
#
# Author
#  - Takashi Nakamoto <takashi.nakamoto@cosylab.com>
#

SCRIPTDIR=$(cd $(dirname $0) && pwd)

ACC=$1
TMP_INI=/tmp/plugin_customization_$$.ini
CSS=${SCRIPTDIR}/css

VALID_ACCS="JPARC PFAR PF LINAC"

JPARC_ADDR_LIST="jparc.kek.jp"
JPARC_ARCHIVE_URL="jdbc:postgresql://localhost:5432/archive"
JPARC_ARCHIVE_NAME="J-PARC"

PFAR_ADDR_LIST="pf-ar.kek.jp"
PFAR_ARCHIVE_URL="xnds://pf-ar.kekjp/archive/cgi/ArchiveDataServer.cgi"
PFAR_ARCHIVE_NAME="PF-AR"

PF_ADDR_LIST="pf.kek.jp"
PF_ARCHIVE_URL="xnds://pf.kekjp/archive/cgi/ArchiveDataServer.cgi"
PF_ARCHIVE_NAME="PF"

LINAC_ADDR_LIST="linac.kek.jp"
LINAC_ARCHIVE_URL="xnds://linac.kekjp/archive/cgi/ArchiveDataServer.cgi"
LINAC_ARCHIVE_NAME="LINAC"

valid=0
ADDR_LIST=""
ARCHIVE_URLS=()
ARCHIVE_NAMES=()

for a in $VALID_ACCS; do
    URL=$(eval 'echo $'$a'_ARCHIVE_URL')
    NAME=$(eval 'echo $'$a'_ARCHIVE_NAME')

    if [ "$a" = "$ACC" ]; then
	ARCHIVE_URLS=(${URL} ${ARCHIVE_URLS[@]})
	ARCHIVE_NAMES=(${NAME} ${ARCHIVE_NAMES[@]})

	ADDR_LIST=$(eval 'echo $'$a'_ADDR_LIST')
	valid=1
    else
	ARCHIVE_URLS=(${ARCHIVE_URLS[@]} ${URL})
	ARCHIVE_NAMES=(${ARCHIVE_NAMES[@]} ${NAME})
    fi
done

if [ $valid -ne 1 ]; then
    echo "Invalid accelerator name: $ACC"
    echo ""
    echo "Valid accelerator names are:"
    for a in $VALID_ACCS; do
	echo " - $a"
    done
    exit 1
fi

for i in `seq ${#ARCHIVE_URLS[@]}`; do
    j=$(expr "$i-1")
    DATABROWSER_URLS="${DATABROWSER_URLS}*${ARCHIVE_URLS[$j]}"
    DATABROWSER_ARCHIVES="${DATABROWSER_ARCHIVES}*${ARCHIVE_NAMES[$j]}|$i|${ARCHIVE_URLS[$j]}"
done

FIRST_CHAR=$(echo "${DATABROWSER_URLS}" | cut -c 1-1)
if [ "${FIRST_CHAR}" = "*" ]; then
    DATABROWSER_URLS=$(echo "${DATABROWSER_URLS}" | cut -c 2-)
fi

FIRST_CHAR=$(echo "${DATABROWSER_ARCHIVES}" | cut -c 1-1)
if [ "${FIRST_CHAR}" = "*" ]; then
    DATABROWSER_ARCHIVES=$(echo "${DATABROWSER_ARCHIVES}" | cut -c 2-)
fi

cat > ${TMP_INI} <<EOF
org.csstudio.platform.libs.epics/addr_list=${ADDR_LIST}

org.csstudio.trends.databrowser2/archives=${DATABROWSER_ARCHIVES}
org.csstudio.trends.databrowser2/urls=${DATABROWSER_URLS}
EOF

${CSS} -vm ${HOME}/jdk1.6.0_27/jre/bin -pluginCustomization ${TMP_INI}
